<?php
defined('ADMIN_WS_API_PATH') or die('Hacking attempt!');

function admin_ws_api_ws_add_methods($arr)
{
  $service = &$arr[0];
  
  $service->addMethod(
      'pwg.groups.getList',
      'ws_groups_getList',
      array(
        'group_id' =>     array('default'=>0),
        'name' =>         array('default'=>null),
        ),
      'Admin only. With default parameters, retrieves a list of all the groups.  Otherwise, filters the list according to the parameters.'
    );

  $service->addMethod(
      'pwg.groups.add',
      'ws_groups_add',
      array(
        'name' =>   array('default'=>null),
        ),
      'Admin only. Creates a group and returns the new group record.'
    );

  $service->addMethod(
      'pwg.groups.delete',
      'ws_groups_delete',
      array(
        'group_id' =>     array('default'=>0),
        ),
      'Admin only. Removes any users from the group and deletes the group. NOTE: Users and Photos are not deleted.'
    );

  $service->addMethod(
      'pwg.groups.rename',
      'ws_groups_rename',
      array(
        'group_id' =>     array('default'=>0),
        'new_name' =>     array('default'=>null),
        ),
      'Admin only. Renames the group.'
    );

  $service->addMethod(
      'pwg.categories.permissions.addGroup',
      'ws_categories_permissions_addGroup',
      array(
        'cat_ids' =>      array('flags'=>WS_PARAM_FORCE_ARRAY),
        'group_id' =>     array('default'=>0),
        ),
      'Admin only. Gives the group permission to access a list of categories (albums). Category_ids must be pipe (|) separated.'
    );

  $service->addMethod(
      'pwg.categories.permissions.deleteGroup',
      'ws_categories_permissions_deleteGroup',
      array(
        'cat_ids' =>      array('flags'=>WS_PARAM_FORCE_ARRAY),
        'group_id' =>     array('default'=>0),
        ),
      'Admin only. Denies the group access to a list of categories (albums). Category_ids must be pipe (|) separated.'
    );

  $service->addMethod(
      'pwg.users.getList',
      'ws_users_getList',
      array(
        'user_id' =>      array('default'=>0),
        'username' =>     array('default'=>null),
        'group_id' =>     array('default'=>0),
        ),
      'Admin only. With default parameters, retrieves a list of all the users.  Otherwise, filters the list according to the parameters.'
    );

  $service->addMethod(
      'pwg.users.add',
      'ws_users_add',
      array(
        'username' =>     array('default'=>null),
        'password' =>     array('default'=>null),
        'email' =>        array('default'=>null),
        ),
      'Admin only. Registers a new user and returns the new user_id.'
    );

  $service->addMethod(
      'pwg.users.updateDetails',
      'ws_users_updateDetails',
      array(
        'username' =>     array('default'=>null),
        'password' =>     array('default'=>null),
        'email' =>        array('default'=>null),
        ),
      'Admin only. Updates the password and email for the given username.  Leave a field (password or email) blank if you do not want to update it.'
    );

  $service->addMethod(
      'pwg.users.delete',
      'ws_users_delete',
      array(
        'user_id' =>      array('default'=>0),
        ),
      'Admin only. Removes the user from any associated groups and deletes the user.  NOTE: Photos owned by this user are not deleted.'
    );

  $service->addMethod(
      'pwg.groups.addUser',
      'ws_groups_addUser',
      array(
        'group_id' =>     array('default'=>0),
        'user_id' =>      array('default'=>0),
        ),
      'Admin only. Adds a user to a group.'
    );

  $service->addMethod(
      'pwg.groups.deleteUser',
      'ws_groups_deleteUser',
      array(
        'user_id' =>      array('default'=>0),
        'group_id' =>     array('default'=>0),
        ),
      'Admin only. Removes the user from the group.'
    );

  $service->addMethod(
      'icy.acls.groups.addAlbum',
      'ws_icy_acls_groups_addAlbum',
      array(
        'groupname' =>    array('default'=>null),
        'cat_id' =>       array('default'=>0),
        ),
      'Admin only. Requires icy_picture_modify plugin. Adds the group to the ACL file, thereby giving the group permission to the category (album).'
    );

  $service->addMethod(
      'icy.acls.groups.addUser',
      'ws_icy_acls_groups_addUser',
      array(
        'username' =>     array('default'=>null),
        'groupname' =>    array('default'=>null),
        ),
      'Admin only. Requires icy_picture_modify plugin. Adds the user to the ACL file, thereby adding the user to the group.'
    );

  $service->addMethod(
      'icy.acls.groups.deleteUser',
      'ws_icy_acls_groups_deleteUser',
      array(
        'username' =>     array('default'=>null),
        'groupname' =>    array('default'=>null),
        ),
      'Admin only. Requires icy_picture_modify plugin. Deletes the user from the ACL file, thereby removing the user from the group.'
    );

}

/**
 * returns groups (web service method)
 */
function ws_groups_getList($params, &$service)
{
  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $query = 'SELECT * FROM '.GROUPS_TABLE.' ';

  $clauseSeparator = " WHERE";

  if (!empty($params['name']))
  {
    $query .= $clauseSeparator.' name = \''.pwg_db_real_escape_string($params['name']).'\'';
    $clauseSeparator = " AND";
  }

  if (!empty($params['group_id']))
  {
    $query .= $clauseSeparator.' id = '.pwg_db_real_escape_string($params['group_id']);
  }

  $query .= ';';

  $result = pwg_query($query);

  $output = array();
  while ($row = pwg_db_fetch_assoc($result))
  {
    $output[] = $row;
  }

  return array('groups' => new PwgNamedArray($output, 'item'));
}

/**
 * adds the group (web service method)
 */
function ws_groups_add($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $validateResult = admin_validate_group_add($params['name']);
  if ($validateResult !== TRUE)
    return $validateResult;

    return admin_group_add($params['name']);
}


function ws_groups_delete($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $validateResult = admin_validate_group_delete($params['group_id']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_group_delete($params['group_id']);
}

function ws_groups_rename($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $validateResult = admin_validate_group_rename($params['group_id'], $params['new_name']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_group_rename($params['group_id'], $params['new_name']);
}

function ws_categories_permissions_addGroup($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $validateResult = admin_validate_categories_permissions_addGroup($params['group_id'],$params['cat_ids']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_categories_permissions_addGroup($params['group_id'], $params['cat_ids']);
}

function ws_categories_permissions_deleteGroup($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $validateResult = admin_validate_categories_permissions_deleteGroup($params['group_id'],$params['cat_ids']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_categories_permissions_deleteGroup($params['group_id'], $params['cat_ids']);
}

/**
 * returns a list of users (web service method)
 */
function ws_users_getList($params, &$service)
{
  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $query = '
  SELECT U.*, UG.group_id FROM '.USERS_TABLE.' U 
  LEFT JOIN '.USER_GROUP_TABLE.' UG ON U.id = UG.user_id ';

  $clauseSeparator = " WHERE";

  if (!empty($params['username']))
  {
    $query .= $clauseSeparator.' username = \''.pwg_db_real_escape_string($params['username']).'\'';
    $clauseSeparator = " AND";
  }

  if (!empty($params['group_id']))
  {
    $query .= $clauseSeparator.' group_id = '.pwg_db_real_escape_string($params['group_id']);
    $clauseSeparator = " AND";
  }

  if (!empty($params['user_id']))
  {
    $query .= $clauseSeparator.' U.id = '.pwg_db_real_escape_string($params['user_id']);
  }

  $query .= ';';

  $result = pwg_query($query);

  $users = array();
  while ($row = pwg_db_fetch_assoc($result))
  {
    $users[] = $row;
  }

  return array('users' => new PwgNamedArray($users, 'item'));
}

/**
 * registers the user
 */
function ws_users_add($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $validateResult = admin_validate_user_add($params['username'],$params['password'],$params['email']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_user_add($params['username'],$params['password'],$params['email']);
}

function ws_users_updateDetails($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $validateResult = admin_validate_user_updateDetails($params['username'],$params['password'],$params['email']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_user_updateDetails($params['username'],$params['password'],$params['email']);
}

function ws_users_delete($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  // $validateResult = admin_validate_user_delete($params['user_id']);
  // if ($validateResult !== TRUE)
  //   return $validateResult;

  return admin_user_delete($params['user_id']);
}

function ws_groups_addUser($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $validateResult = admin_validate_groups_addUser($params['group_id'],$params['user_id']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_groups_addUser($params['group_id'],$params['user_id']);
}

function ws_groups_deleteUser($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $validateResult = admin_validate_groups_deleteUser($params['group_id'],$params['user_id']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_groups_deleteUser($params['group_id'],$params['user_id']);
}

function ws_icy_acls_groups_addAlbum($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $conf_path = PHPWG_ROOT_PATH.PWG_LOCAL_DIR.'config/icy_acl.zml';
  $validateResult = admin_validate_icy_acls_groups_addAlbum($params['groupname'],$params['cat_id']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_icy_acls_groups_addAlbum($params['groupname'],$params['cat_id']);
} 

function ws_icy_acls_groups_addUser($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $conf_path = PHPWG_ROOT_PATH.PWG_LOCAL_DIR.'config/icy_acl.zml';
  $validateResult = admin_validate_icy_acls_groups_addUser($params['username'],$params['groupname']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_icy_acls_groups_addUser($params['username'],$params['groupname']);
} 

function ws_icy_acls_groups_deleteUser($params, &$service)
{
  if (!$service->isPost())
  {
    return new PwgError(405, "This method requires HTTP POST");
  }

  if (!is_admin())
  {
    return new PwgError(401, 'Access denied');
  }

  $conf_path = PHPWG_ROOT_PATH.PWG_LOCAL_DIR.'config/icy_acl.zml';
  $validateResult = admin_validate_icy_acls_groups_deleteUser($params['username'],$params['groupname']);
  if ($validateResult !== TRUE)
    return $validateResult;

  return admin_icy_acls_groups_deleteUser($params['username'],$params['groupname']);
} 

?>